@@ -1,4 +1,4 @@
-/*  $Id: ckpasswd.c 7585 2006-11-21 09:37:51Z eagle $
+/*  $Id: ckpasswd.c 9587 2013-12-14 20:10:48Z iulius $
 **
 **  The default username/password authenticator.
 **
@@ -21,20 +21,28 @@
 #if HAVE_CRYPT_H
 # include <crypt.h>
 #endif
+
 #include <fcntl.h>
 #include <pwd.h>
 #include <grp.h>
 
-#if defined(HAVE_DBM) || defined(HAVE_BDB_DBM)
-# if HAVE_NDBM_H
-#  include <ndbm.h>
-# elif HAVE_BDB_DBM
+
+/*
+**  If compiling with Berkeley DB, use its ndbm compatibility layer
+**  in preference to other libraries.
+*/
+#if defined(HAVE_DBM) || defined(HAVE_BDB_NDBM)
+# if HAVE_BDB_NDBM
 #  define DB_DBM_HSEARCH 1
 #  include <db.h>
-# elif HAVE_GDBM_NDBM_H
-#  include <gdbm-ndbm.h>
+# elif HAVE_NDBM_H
+#  include <ndbm.h>
 # elif HAVE_DB1_NDBM_H
 #  include <db1/ndbm.h>
+# elif HAVE_GDBM_SLASH_NDBM_H
+#  include <gdbm/ndbm.h>
+# elif HAVE_GDBM_HYPHEN_NDBM_H
+#  include <gdbm-ndbm.h>
 # endif
 # define OPT_DBM "d:"
 #else
@@ -148,7 +156,7 @@ auth_pam(const char *username, char *password)
 **  password, if found, is returned as a newly allocated string; otherwise,
 **  NULL is returned.
 */
-#if !(defined(HAVE_DBM) || defined(HAVE_BDB_DBM))
+#if !(defined(HAVE_DBM) || defined(HAVE_BDB_NDBM))
 static char *
 password_dbm(char *user UNUSED, const char *file UNUSED)
 {
@@ -177,7 +185,7 @@ password_dbm(char *name, const char *file)
     dbm_close(database);
     return password;
 }
-#endif /* HAVE_DBM || HAVE_BDB_DBM */
+#endif /* HAVE_DBM || HAVE_BDB_NDBM */
 
 
 /*
