@@ -1,4 +1,4 @@
-/*  $Id: ninpaths.c 7399 2005-08-26 07:39:49Z eagle $
+/*  $Id: ninpaths.c 9584 2013-12-12 20:24:20Z iulius $
 **
 **  New inpaths reporting program.
 **
@@ -147,7 +147,7 @@ writedump(FILE *f)
     }
     fprintf(f, "!!NINP " VERSION " %lu %lu %ld %ld %ld\n",
             (unsigned long) starttime, (unsigned long) time(NULL), sites,
-            total, (long)(atimes/total)+starttime);
+            total, (long) ((atimes/total) + starttime));
     n=j=0;
     /* write the S-records (hosts), numbering them in the process */
     for (i=0; i<HASH_TBL; ++i)
@@ -368,11 +368,11 @@ procpaths(FILE *f)
 
     while (fgets(buf, sizeof(buf), f)) {
 	c=buf;
-	if (!strncmp(c, "Path: ", 6))
+	if (!strncasecmp(c, "Path: ", 6))
 	    c+=6;
 	/* find end of line. Some broken newsreaders preload Path with
 	   a name containing spaces. Chop off those entries. */
-	for (ce=c; *ce && !CTYPE(isspace, *ce); ++ce);
+	for (ce=c; *ce && !isspace((unsigned char) *ce); ++ce);
 	if (!*ce) {
 	    /* bogus line */
 	    v=0;
