@@ -24,10 +24,10 @@
 
 #include "inn/innconf.h"
 #include "inn/messages.h"
-#include "libinn.h"
-#include "paths.h"
-#include "storage.h"
-#include "ov.h"
+#include "inn/libinn.h"
+#include "inn/paths.h"
+#include "inn/storage.h"
+#include "inn/ov.h"
 
 #include "../storage/ovdb/ovdb.h"
 #include "../storage/ovdb/ovdb-private.h"
@@ -37,7 +37,7 @@
 int
 main(int argc UNUSED, char **argv UNUSED)
 {
-    die("BerkeleyDB support not compiled");
+    die("Berkeley DB support not compiled");
 }
 
 #else /* USE_BERKELEY_DB */
@@ -67,6 +67,7 @@ struct reader {
     int bufpos;
     void *buf;
     time_t lastactive;
+    void *currentsearch;
 };
 
 static struct reader *readertab;
@@ -148,7 +149,7 @@ do_groupstats(struct reader *r)
     char *group = (char *)(r->buf) + sizeof(struct rs_cmd);
     reply = xmalloc(sizeof(struct rs_groupstats));
 
-    /*syslog(LOG_DEBUG, "OVDB: rs: do_groupstats '%s'", group);*/
+    debug("OVDB: rs: do_groupstats '%s'", group);
     if(ovdb_groupstats(group, &reply->lo, &reply->hi, &reply->count, &reply->flag)) {
 	reply->status = CMD_GROUPSTATS;
     	reply->aliaslen = 0;
@@ -170,13 +171,19 @@ do_opensrch(struct reader *r)
     char *group = (char *)(r->buf) + sizeof(struct rs_cmd);
     reply = xmalloc(sizeof(struct rs_opensrch));
 
-    /*syslog(LOG_DEBUG, "OVDB: rs: do_opensrch '%s' %d %d", group, cmd->artlo, cmd->arthi);*/
+    debug("OVDB: rs: do_opensrch '%s' %d %d", group, cmd->artlo, cmd->arthi);
 
-    reply->handle = ovdb_opensearch(group, cmd->artlo, cmd->arthi);
-    if(reply->handle == NULL) {
-    	reply->status = CMD_OPENSRCH | RPLY_ERROR;
+    if(r->currentsearch != NULL) {
+	/* can only open one search at a time */
+	reply->status = CMD_OPENSRCH | RPLY_ERROR;
     } else {
-    	reply->status = CMD_OPENSRCH;
+	reply->handle = ovdb_opensearch(group, cmd->artlo, cmd->arthi);
+	if(reply->handle == NULL) {
+	    reply->status = CMD_OPENSRCH | RPLY_ERROR;
+	} else {
+	    reply->status = CMD_OPENSRCH;
+	}
+	r->currentsearch = reply->handle;
     }
     free(r->buf);
     r->buf = reply;
@@ -226,6 +233,7 @@ do_closesrch(struct reader *r)
     r->buf = NULL;
     r->bufpos = r->buflen = 0;
     r->mode = MODE_READ;
+    r->currentsearch = NULL;
 }
 
 static void
@@ -236,7 +244,7 @@ do_artinfo(struct reader *r)
     char *group = (char *)(r->buf) + sizeof(struct rs_cmd);
     TOKEN token;
 
-    /*syslog(LOG_DEBUG, "OVDB: rs: do_artinfo: '%s' %d", group, cmd->artlo);*/
+    debug("OVDB: rs: do_artinfo: '%s' %d", group, cmd->artlo);
     if(ovdb_getartinfo(group, cmd->artlo, &token)) {
     	reply = xmalloc(sizeof(struct rs_artinfo));
 	reply->status = CMD_ARTINFO;
@@ -370,12 +378,17 @@ static void
 newclient(int fd)
 {
     struct reader *r;
+    int i;
 
     nonblocking(fd, 1);
 
     if(numreaders >= readertablen) {
     	readertablen += 50;
         readertab = xrealloc(readertab, readertablen * sizeof(struct reader));
+        for(i = numreaders; i < readertablen; i++) {
+	    readertab[i].mode = MODE_CLOSED;
+	    readertab[i].buf = NULL;
+	}
     }
 
     r = &(readertab[numreaders]);
@@ -387,6 +400,7 @@ newclient(int fd)
     r->bufpos = 0;
     r->buf = xstrdup(OVDB_SERVER_BANNER);
     r->lastactive = now;
+    r->currentsearch = NULL;
 
     handle_write(r);
 }
@@ -403,6 +417,10 @@ delclient(int which)
     if(r->buf != NULL) {
     	free(r->buf);
     }
+    if(r->currentsearch != NULL) {
+	ovdb_closesearch(r->currentsearch);
+	r->currentsearch = NULL;
+    }
 
     /* numreaders will get decremented by the calling function */
     for(i = which; i < numreaders-1; i++)
@@ -432,6 +450,7 @@ serverproc(int me)
     xsignal_norestart(SIGTERM, sigfunc);
     xsignal_norestart(SIGHUP, sigfunc);
     xsignal_norestart(SIGUSR1, childsig);
+    xsignal(SIGPIPE, SIG_IGN);
 
     numreaders = lastnumreaders = 0;
     if(ovdb_conf.maxrsconn) {
@@ -517,7 +536,7 @@ serverproc(int me)
 
 	for(i = 0; i < numreaders; i++) {
 	    if(readertab[i].mode == MODE_CLOSED
-		  || readertab[i].lastactive + CLIENT_TIMEOUT < now) {
+		  || (time_t) (readertab[i].lastactive + CLIENT_TIMEOUT) < now) {
 	    	delclient(i);
 		numreaders--;
 		i--;
@@ -629,17 +648,18 @@ main(int argc, char *argv[])
 
     nonblocking(listensock, 1);
 
+    memset(&sa, 0, sizeof sa);
 #ifdef HAVE_UNIX_DOMAIN_SOCKETS
     sa.sun_family = AF_UNIX;
     path = concatpath(innconf->pathrun, OVDB_SERVER_SOCKET);
     strlcpy(sa.sun_path, path, sizeof(sa.sun_path));
     unlink(sa.sun_path);
     free(path);
-    ret = bind(listensock, (struct sockaddr *)&sa, sizeof sa);
+    ret = bind(listensock, (struct sockaddr *)&sa, SUN_LEN(&sa));
 #else
     sa.sin_family = AF_INET;
     sa.sin_port = htons(OVDB_SERVER_PORT);
-    sa.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+    sa.sin_addr.s_addr = htonl(0x7f000001UL);
     
     ret = bind(listensock, (struct sockaddr *)&sa, sizeof sa);
 
