@@ -1,4 +1,4 @@
-/*  $Id: art.c 9024 2010-03-21 16:49:30Z iulius $
+/*  $Id: art.c 9521 2013-07-31 17:27:33Z iulius $
 **
 **  Article-processing.
 */
@@ -506,7 +506,7 @@ ARTstore(CHANNEL *cp)
 	  iov[iovcnt++].iov_len = data->XrefLength - 2;
 	  arth.len += data->XrefLength - 2;
 	  /* next to write */
-	  /* this points where trailing "\r\n" of orginal Xref header exists */
+	  /* this points where trailing "\r\n" of original Xref: header exists */
 	  p = HDR(HDR__XREF) + HDR_LEN(HDR__XREF);
 	}
 	break;
@@ -649,7 +649,7 @@ ARTcheckheader(CHANNEL *cp, int size)
   }
 
   /* See if this is a system header.  A fairly tightly-coded binary search. */
-  c = islower((unsigned char) *header) ? toupper(*header) : *header;
+  c = islower((unsigned char) *header) ? toupper((unsigned char) *header) : *header;
   for (*colon = '\0', tp = ARTheadertree; tp; ) {
     if ((i = c - tp->Name[0]) == 0 && (i = strcasecmp(header, tp->Name)) == 0)
       break;
@@ -683,13 +683,26 @@ ARTcheckheader(CHANNEL *cp, int size)
     cp->Data.BytesHeader = header;
   hc = &hc[i];
   if (hc->Length != 0) {
-    /* Duplicated. */
-    hc->Length = -1;
+    /* Duplicated required header.
+     * We do not check every header because they would otherwise disappear
+     * from the overview.
+     * The content of the first occurrence must be returned by HDR and OVER
+     * according to RFC 3977. */
+    if (ARTheaders[i].Type == HTreq) {
+      hc->Length = -1;
+    }
   } else {
     /* We need to remove leading and trailing spaces for
      * message-IDs; otherwise, history hashes may not be
-     * correctly computed. */
-    if (i == HDR__MESSAGE_ID || i == HDR__SUPERSEDES) {
+     * correctly computed.
+     * We also do it for Xref: header fields because
+     * a few parts of the code currently assume that no
+     * leading whitespace exists for this header (when
+     * parsing it to find the newsgroups in which the article
+     * is stored).  Only when INN does not generate it (that is
+     * to say in xrefslave mode). */
+    if (i == HDR__MESSAGE_ID || i == HDR__SUPERSEDES ||
+        (innconf->xrefslave && i == HDR__XREF)) {
       for (p = colon + 1 ; (p < header + size - 2) &&
            (ISWHITE(*p)) ; p++);
       hc->Value = p;
@@ -724,7 +737,7 @@ ARTprepare(CHANNEL *cp)
   }
   data->Lines = data->HeaderLines = data->CRwithoutLF = data->LFwithoutCR = 0;
   data->DotStuffedLines = 0;
-  data->CurHeader = data->LastCRLF = data->Body = cp->Start;
+  data->CurHeader = data->Body = cp->Start;
   data->BytesHeader = NULL;
   data->Feedsite = "?";
   data->FeedsiteLength = strlen(data->Feedsite);
@@ -801,13 +814,13 @@ ARTparseheader(CHANNEL *cp)
     struct buffer *bp = &cp->In;
     ARTDATA *data = &cp->Data;
     size_t i;
-    unsigned long length;
 
     for (i = cp->Next; i < bp->used; i++) {
         if (bp->data[i] == '\0')
             ARTerror(cp, "Nul character in header");
-        if (bp->data[i] == '\n')
+        if (bp->data[i] == '\n') {
             data->LFwithoutCR++;
+        }
         if (bp->data[i] != '\r')
             continue;
 
@@ -842,12 +855,6 @@ ARTparseheader(CHANNEL *cp)
             cp->Next = i + 5;
             return;
         } else if (bp->data[i + 1] == '\n') {
-            length = i - data->LastCRLF - 1;
-            if (data->LastCRLF == cp->Start)
-                length++;
-            if (length > MAXHEADERSIZE)
-                ARTerror(cp, "Header line too long (%lu bytes)", length);
-
             /* Be a little tricky here.  Normally, the headers end at the
                first occurrence of \r\n\r\n, so since we've seen \r\n, we want
                to advance i and then look to see if we have another one.  The
@@ -871,7 +878,6 @@ ARTparseheader(CHANNEL *cp)
                 ARTparsebody(cp);
 		return;
             }
-            data->LastCRLF = i - 1;
         } else {
             data->CRwithoutLF++;
         }
@@ -933,7 +939,7 @@ ARTparsebody(CHANNEL *cp)
             if (bp->data[i + 1] == '.')
                 data->DotStuffedLines++;
         } else {
-            data->LFwithoutCR++;
+            data->CRwithoutLF++;
         }
     }
     cp->Next = i;
@@ -1025,17 +1031,34 @@ ARTclean(ARTDATA *data, char *buff, bool ihave)
     }
   }
 
-  /* Is article too old? */
-  /* assumes Date header is required header */
+  /* Is the article too old? */
+  /* Assumes the Date: header is a required header.
+   * Check the presence of the Injection-Date: header field, which will
+   * override the value of the time the article was posted. */
   p = HDR(HDR__DATE);
   data->Posted = parsedate_rfc5322_lax(p);
+
   if (data->Posted == (time_t) -1) {
     sprintf(buff, "%d Bad \"Date\" header -- \"%s\"",
             ihave ? NNTP_FAIL_IHAVE_REJECT : NNTP_FAIL_TAKETHIS_REJECT,
-      MaxLength(p, p));
+            MaxLength(p, p));
     TMRstop(TMR_ARTCLEAN);
     return false;
   }
+
+  if (HDR_FOUND(HDR__INJECTION_DATE)) {
+    p = HDR(HDR__INJECTION_DATE);
+    data->Posted = parsedate_rfc5322_lax(p);
+
+    if (data->Posted == (time_t) -1) {
+      sprintf(buff, "%d Bad \"Injection-Date\" header -- \"%s\"",
+              ihave ? NNTP_FAIL_IHAVE_REJECT : NNTP_FAIL_TAKETHIS_REJECT,
+              MaxLength(p, p));
+      TMRstop(TMR_ARTCLEAN);
+      return false;
+    }
+  }
+
   if (innconf->artcutoff != 0) {
       long cutoff = innconf->artcutoff * 24 * 60 * 60;
 
@@ -1048,7 +1071,7 @@ ARTclean(ARTDATA *data, char *buff, bool ihave)
       }
   }
   if (data->Posted > Now.tv_sec + DATE_FUZZ) {
-    sprintf(buff, "%d Article posted in the future -- \"%s\"",
+    sprintf(buff, "%d Article injected or posted in the future -- \"%s\"",
             ihave ? NNTP_FAIL_IHAVE_REJECT : NNTP_FAIL_TAKETHIS_REJECT,
             MaxLength(p, p));
     TMRstop(TMR_ARTCLEAN);
@@ -1397,7 +1420,7 @@ ARTpoisongroup(char *name)
 }
 
 /*
-** Assign article numbers to the article and create the Xref line.
+** Assign article numbers to the article and create the Xref: header field.
 ** If we end up not being able to write the article, we'll get "holes"
 ** in the directory and active file.
 */
@@ -1409,7 +1432,7 @@ ARTassignnumbers(ARTDATA *data)
   NEWSGROUP	*ngp;
 
   if (data->XrefBufLength == 0) {
-    data->XrefBufLength = MAXHEADERSIZE * 2 + 1;
+    data->XrefBufLength = MED_BUFFER * 2 + 1;
     data->Xref = xmalloc(data->XrefBufLength);
     strncpy(data->Xref, Path.data, Path.used - 1);
   }
@@ -1429,14 +1452,16 @@ ARTassignnumbers(ARTDATA *data)
       continue;
     }
     ngp->Filenum = ngp->Last;
-    /*  len  ' ' "news_groupname"  ':' "#" "\r\n" */
-    if (len + 1 + ngp->NameLength + 1 + 10 + 2 > data->XrefBufLength) {
-      data->XrefBufLength += MAXHEADERSIZE;
+    /*  len  ' ' "news_groupname"  ':' "#" "\r\n"
+        plus an extra 2 bytes for "\r\n" in case of a continuation line. */
+    if (len + 1 + ngp->NameLength + 1 + ARTNUMPRINTSIZE + 2 + 2 > data->XrefBufLength) {
+      data->XrefBufLength += MED_BUFFER;
       data->Xref = xrealloc(data->Xref, data->XrefBufLength);
       p = data->Xref + len;
     }
-    if (linelen + 1 + ngp->NameLength + 1 + 10 > MAXHEADERSIZE) {
-      /* line exceeded */
+    /* Trailing CRLF is counted in the maximum length. */
+    if (linelen + 1 + ngp->NameLength + 1 + ARTNUMPRINTSIZE + 2 > MAXARTLINELENGTH) {
+      /* Line exceeded. */
       sprintf(p, "\r\n %s:%lu", ngp->Name, ngp->Filenum);
       buflen = strlen(p);
       linelen = buflen - 2;
@@ -1449,17 +1474,18 @@ ARTassignnumbers(ARTDATA *data)
     p += buflen;
   }
   /* p[0] is replaced with '\r' to be wireformatted when stored.  p[1] needs to
-     be '\n' */
+     be '\n'.  We have enough place to modify p here (checked during the
+     reallocation above). */
   p[0] = '\r';
   p[1] = '\n';
-  /* data->XrefLength includes trailing "\r\n" */
+  /* data->XrefLength includes trailing "\r\n". */
   data->XrefLength = len + 2;
   data->Replic = q + 1;
   data->ReplicLength = len - (q + 1 - data->Xref);
 }
 
 /*
-**  Parse the data from the xref header and assign the numbers.
+**  Parse the data from the Xref: header and assign the numbers.
 **  This involves replacing the GroupPointers entries.
 */
 static bool
@@ -1535,18 +1561,17 @@ ARTxrefslave(ARTDATA *data)
 
 /*
 **  Return true if a list of strings has a specific one.  This is a
-**  generic routine, but is used for seeing if a host is in the Path line.
+**  generic routine, but is used for seeing if a host is in the Path: line.
 */
 static bool
 ListHas(const char **list, const char *p)
 {
-  const char	*q;
-  char		c;
-
-  for (c = *p; (q = *list) != NULL; list++)
-    if (strcasecmp(p, q) == 0)
-      return true;
-  return false;
+    size_t i;
+     
+    for (i = 0; list[i] != NULL; i++)
+        if (strcasecmp(p, list[i]) == 0)
+            return true;
+    return false;
 }
 
 /*
@@ -1819,7 +1844,7 @@ ARTmakeoverview(CHANNEL *cp)
   }
 
   /* Setup. */
-  buffer_resize(overview, MAXHEADERSIZE);
+  buffer_resize(overview, MED_BUFFER);
   buffer_set(overview, "", 0);
 
   /* Write the data, a field at a time. */
@@ -1931,20 +1956,23 @@ ARTpost(CHANNEL *cp)
   bool		IsControl = false;
   bool		Filtered = false;
   bool          ihave;
-  struct buffer	*article;
   HASH		hash;
   TOKEN		token;
   char		*groupbuff[2];
 #if defined(DO_PERL) || defined(DO_PYTHON)
+  struct buffer *article;
   char		*filterrc;
-#endif /* defined(DO_PERL) || defined(DO_PYTHON) */
+#endif
   OVADDRESULT	result;
 
   /* Check whether we are receiving the article via IHAVE or TAKETHIS. */
   ihave = (cp->Sendid.size > 3) ? false : true;
 
-  /* Preliminary clean-ups. */
+#if defined(DO_PERL) || defined(DO_PYTHON)
   article = &cp->In;
+#endif
+
+  /* Preliminary clean-ups. */
   artclean = ARTclean(data, cp->Error, ihave);
 
   /* We have not parsed the Path: header yet.  We do not check for logipaddr
@@ -2186,7 +2214,7 @@ ARTpost(CHANNEL *cp)
     strlcpy(ControlWord, HDR(HDR__CONTROL), sizeof(ControlWord));
     for (p = ControlWord; *p && !ISWHITE(*p); p++)
       if (isupper((unsigned char) *p))
-	*p = tolower(*p);
+	*p = tolower((unsigned char) *p);
     *p = '\0';
     LikeNewgroup = (strcasecmp(ControlWord, "newgroup") == 0
                     || strcasecmp(ControlWord, "rmgroup") == 0);
@@ -2559,7 +2587,7 @@ ARTpost(CHANNEL *cp)
                data->LFwithoutCR);
     else
       snprintf(cp->Error, sizeof(cp->Error),
-               "Article accepted but includes CR without LF(%d) and LF withtout CR(%d)",
+               "Article accepted but includes CR without LF(%d) and LF without CR(%d)",
                data->CRwithoutLF, data->LFwithoutCR);
     /* We have another ARTlog() for the same article just after. */
     ARTlog(data, ART_STRSTR, cp->Error);
