@@ -1,4 +1,4 @@
-/*  $Id: snprintf.c 8838 2009-12-03 21:11:00Z iulius $
+/*  $Id: snprintf.c 9352 2011-08-19 15:16:19Z iulius $
 **
 **  Replacement for a missing snprintf or vsnprintf.
 **
@@ -107,11 +107,8 @@
 #define VA_SHIFT(v,t)  ;   /* no-op for ANSI */
 #define VA_END          va_end(ap)
 
-#ifdef HAVE_LONG_DOUBLE
+/* Assume all compilers support long double, per Autoconf documentation. */
 #define LDOUBLE long double
-#else
-#define LDOUBLE double
-#endif
 
 #ifdef HAVE_LONG_LONG_INT
 # define LLONG long long
