@@ -1,4 +1,4 @@
-/*  $Id: misc.c 8061 2008-09-27 10:04:47Z iulius $
+/*  $Id: misc.c 9497 2013-06-25 17:40:59Z iulius $
 **
 **  Miscellaneous support routines.
 */
@@ -26,71 +26,6 @@ extern bool nnrpd_starttls_done;
 #endif 
 
 
-/*
-**  Parse a string into a NULL-terminated array of words; return number
-**  of words.  If argvp isn't NULL, it and what it points to will be freed.
-*/
-int
-Argify(char *line, char ***argvp)
-{
-    char	**argv;
-    char	*p;
-
-    if (*argvp != NULL) {
-	free(*argvp[0]);
-	free(*argvp);
-    }
-
-    /* Copy the line, which we will split up. */
-    while (ISWHITE(*line))
-	line++;
-    p = xstrdup(line);
-
-    /* Allocate worst-case amount of space. */
-    for (*argvp = argv = xmalloc((strlen(p) + 2) * sizeof(char *)); *p; ) {
-	/* Mark start of this word, find its end. */
-	for (*argv++ = p; *p && !ISWHITE(*p); )
-	    p++;
-	if (*p == '\0')
-	    break;
-
-	/* Nip off word, skip whitespace. */
-	for (*p++ = '\0'; ISWHITE(*p); )
-	    p++;
-    }
-    *argv = NULL;
-    return argv - *argvp;
-}
-
-
-/*
-**  Take a vector which Argify made and glue it back together with
-**  spaces between each element.  Returns a pointer to dynamic space.
-*/
-char *
-Glom(char **av)
-{
-    char	**v;
-    int	i;
-    char		*save;
-
-    /* Get space. */
-    for (i = 0, v = av; *v; v++)
-	i += strlen(*v) + 1;
-    i++;
-
-    save = xmalloc(i);
-    save[0] = '\0';
-    for (v = av; *v; v++) {
-	if (v > av)
-            strlcat(save, " ", i);
-        strlcat(save, *v, i);
-    }
-
-    return save;
-}
-
-
 /*
 **  Match a list of newsgroup specifiers against a list of newsgroups.
 **  func is called to see if there is a match.
@@ -136,9 +71,9 @@ PERMartok(void)
     if (!PERMspecified)
 	return false;
 
-    if ((p = GetHeader("Xref")) == NULL) {
+    if ((p = GetHeader("Xref", true)) == NULL) {
 	/* In case article does not include Xref:. */
-	if ((p = GetHeader("Newsgroups")) != NULL) {
+	if ((p = GetHeader("Newsgroups", true)) != NULL) {
 	    if (!NGgetlist(&grplist, p))
 		/* No newgroups or null entry. */
 		return true;
@@ -246,7 +181,7 @@ InitBackoffConstants(void)
   /* Read the runtime config file to get parameters. */
 
   if ((PERMaccessconf->backoff_db == NULL) ||
-    !(PERMaccessconf->backoff_k >= 0L && PERMaccessconf->backoff_postfast >= 0L && PERMaccessconf->backoff_postslow >= 1L))
+    !(PERMaccessconf->backoff_postslow >= 1L))
     return;
 
   /* Need this database for backing off. */
@@ -349,7 +284,8 @@ LockPostRec(char *path)
     /* No lock.  See if the file is there. */
     if (stat(lockname, &st) < 0) {
       syslog(L_ERROR, "%s cannot stat lock file %s", Client.host, strerror(errno));
-      if (statfailed++ > 5) return(0);
+      if (statfailed++ > 5)
+          return(0);
       continue;
     }
 
@@ -357,7 +293,8 @@ LockPostRec(char *path)
      * PERMaccessconf->backoff_postslow, remove it. */
     statfailed = 0;
     time(&now);
-    if (now < st.st_ctime + PERMaccessconf->backoff_postslow) continue;
+    if (now < (time_t) (st.st_ctime + PERMaccessconf->backoff_postslow))
+        continue;
     syslog(L_ERROR, "%s removing stale lock file %s", Client.host, lockname);
     unlink(lockname);
   }
@@ -398,6 +335,7 @@ GetPostRecord(char *path, long *lastpost, long *lastsleep, long *lastn)
      if (fgets(buff,SMBUF,fp) == NULL) {
        syslog(L_ERROR, "%s Error reading '%s': %s",
               Client.host, path, strerror(errno));
+       fclose(fp);
        return 0;
      }
      *lastpost = atol(buff);
@@ -405,6 +343,7 @@ GetPostRecord(char *path, long *lastpost, long *lastsleep, long *lastn)
      if ((s = strchr(buff,',')) == NULL) {
        syslog(L_ERROR, "%s bad data in postrec file: '%s'",
               Client.host, buff);
+       fclose(fp);
        return 0;
      }
      s++; *lastsleep = atol(s);
@@ -412,6 +351,7 @@ GetPostRecord(char *path, long *lastpost, long *lastsleep, long *lastn)
      if ((s = strchr(s,',')) == NULL) {
        syslog(L_ERROR, "%s bad data in postrec file: '%s'",
               Client.host, buff);
+       fclose(fp);
        return 0;
      }
      s++; *lastn = atol(s);
@@ -451,7 +391,7 @@ RateLimit(long *sleeptime, char *path)
 
      now = time(NULL);
      prevpost = 0L; prevsleep = 0L; prevn = 0L; n = 0L;
-     if (!GetPostRecord(path,&prevpost,&prevsleep,&prevn)) {
+     if (!GetPostRecord(path, &prevpost, &prevsleep, &prevn)) {
        syslog(L_ERROR, "%s can't get post record: %s",
               Client.host, strerror(errno));
        return 0;
@@ -462,7 +402,7 @@ RateLimit(long *sleeptime, char *path)
        prevn = 0L;
      if (prevsleep < 0L)
        prevsleep = 0L;
-     if (prevsleep > PERMaccessconf->backoff_postfast)
+     if ((unsigned long) prevsleep > PERMaccessconf->backoff_postfast)
        prevsleep = PERMaccessconf->backoff_postfast;
      
       /* Compute the new sleep time. */
@@ -477,12 +417,12 @@ RateLimit(long *sleeptime, char *path)
                 Client.host,n);
          n = 0L;
        }
-       if (n < PERMaccessconf->backoff_postfast) {
-         if (prevn >= PERMaccessconf->backoff_trigger) {
+       if ((unsigned long) n < PERMaccessconf->backoff_postfast) {
+         if ((unsigned long) prevn >= PERMaccessconf->backoff_trigger) {
            *sleeptime = 1 + (prevsleep * PERMaccessconf->backoff_k);
          } 
-       } else if (n < PERMaccessconf->backoff_postslow) {
-         if (prevn >= PERMaccessconf->backoff_trigger) {
+       } else if ((unsigned long) n < PERMaccessconf->backoff_postslow) {
+         if ((unsigned long) prevn >= PERMaccessconf->backoff_trigger) {
            *sleeptime = prevsleep;
          }
        } else {
@@ -491,15 +431,17 @@ RateLimit(long *sleeptime, char *path)
        prevn++;
      }
 
-     *sleeptime = ((*sleeptime) > PERMaccessconf->backoff_postfast) ? PERMaccessconf->backoff_postfast : (*sleeptime);
+     *sleeptime = ((*sleeptime) > (long) PERMaccessconf->backoff_postfast) ?
+        (long) PERMaccessconf->backoff_postfast : (*sleeptime);
      /* This ought to trap this bogon. */
      if ((*sleeptime) < 0L) {
-	syslog(L_ERROR,"%s Negative sleeptime detected: %ld, prevsleep: %ld, N: %ld",Client.host,*sleeptime,prevsleep,n);
+	syslog(L_ERROR,"%s Negative sleeptime detected: %ld, prevsleep: %ld, N: %ld",
+               Client.host, *sleeptime, prevsleep, n);
 	*sleeptime = 0L;
      }
   
      /* Store the postrecord. */
-     if (!StorePostRecord(path,now,*sleeptime,prevn)) {
+     if (!StorePostRecord(path, now, *sleeptime, prevn)) {
        syslog(L_ERROR, "%s can't store post record: %s", Client.host, strerror(errno));
        return 0;
      }
@@ -524,7 +466,8 @@ CMDstarttls(int ac UNUSED, char *av[] UNUSED)
 
     /* If the client is already authenticated, STARTTLS is not possible. */
     if (PERMauthorized && !PERMneedauth && !PERMcanauthenticate) {
-        Reply("%d Already authenticated\r\n", NNTP_ERR_ACCESS);
+        Reply("%d Already authenticated without the use of a security layer\r\n",
+              NNTP_ERR_ACCESS);
         return;
     }
 
@@ -578,5 +521,8 @@ CMDstarttls(int ac UNUSED, char *av[] UNUSED)
         GRPcount = 0;
         PERMgroupmadeinvalid = false;
     }
+
+    /* Reset our read buffer so as to prevent plaintext command injection. */
+    line_reset(&NNTPline);
 }
 #endif /* HAVE_SSL */
