@@ -1,4 +1,4 @@
-/*  $Id: buffindexed.c 8425 2009-04-24 19:25:11Z iulius $
+/*  $Id: buffindexed.c 9024 2010-03-21 16:49:30Z iulius $
 **
 **  Overview buffer and index method.
 */
@@ -114,7 +114,7 @@ typedef struct _OVBUFF {
 						   header */
   void *		bitfield;		/* Bitfield for ovbuff block in
 						   use */
-  int			dirty;			/* OVBUFFHEAD dirty count */
+  unsigned long         dirty;                  /* OVBUFFHEAD dirty count */
   struct _OVBUFF	*next;			/* next ovbuff */
   int			nextchunk;		/* next chunk */
   smcd_t		*smc;			/* shared mem control data */
@@ -430,7 +430,7 @@ static bool ovbuffread_config(void) {
     while (1) {
       if (*from && *from == '\\' && *(from + 1) == '\n') {
 	from += 2;	/* Skip past backslash+newline */
-	while (*from && isspace((int)*from))
+	while (*from && isspace((unsigned char) *from))
 	  from++;
 	continue;
       }
@@ -517,7 +517,7 @@ static off_t hex2offt(char *hex) {
 	break;
       }
       n += (*hex - diff);
-      if (isalnum((int)*(hex + 1)))
+      if (isalnum((unsigned char) *(hex + 1)))
 	n <<= 4;
     }
     return n;
@@ -768,7 +768,7 @@ static void ovnextblock(OVBUFF *ovbuff) {
 
   table = ((ULONG *) ovbuff->bitfield + (OV_BEFOREBITF / sizeof(long)));
 
-  /* For tighter placement look for unused block from the begining on
+  /* For tighter placement look for unused block from the beginning on
    * every run. */
   for (i = 0 ; i < last ; i++) {
     if ((table[i] ^ ~0) != 0)
@@ -1715,7 +1715,7 @@ ovgroupmmap(GROUPENTRY *ge, ARTNUM low, ARTNUM high, bool needov)
   }
   if (count == 0)
     return true;
-  if (count * OV_BLOCKSIZE > innconf->keepmmappedthreshold * 1024)
+  if ((unsigned) count * OV_BLOCKSIZE > innconf->keepmmappedthreshold * 1024)
     /* large retrieval, mmap is done in ovsearch() */
     return true;
   /* Data blocks are being mmapped, not copied. */
@@ -2321,7 +2321,7 @@ void buffindexed_close(void) {
   /* sync the bit field */
   ovbuff = ovbufftab;
   for (; ovbuff != (OVBUFF *)NULL; ovbuff = ovbuffnext) {
-    if (ovbuff->dirty) {
+    if (ovbuff->dirty != 0) {
       ovbuff->dirty = OVBUFF_SYNC_COUNT + 1;
       ovflushhead(ovbuff);
     }
@@ -2374,7 +2374,7 @@ main(int argc, char **argv) {
   }
   fprintf(stdout, "GROUPheader->freelist.recno is %d(0x%08x)\n", GROUPheader->freelist.recno, GROUPheader->freelist.recno);
   group = argv[1];
-  if (CTYPE(isdigit, *group)) {
+  if (isdigit((unsigned char) *group)) {
     gloc.recno = atoi(group);
     ge = &GROUPentries[gloc.recno];
     fprintf(stdout, "left articles are %d for %d, last expiry is %ld\n", ge->count, gloc.recno, (long) ge->expired);
