@@ -1,12 +1,9 @@
-/*  $Id: buffindexed.h 9782 2015-01-07 21:34:22Z iulius $
-*/
-
 #ifndef _BUFFINDEXED_H_
 #define _BUFFINDEXED_H_
 
-#include "config.h"
-
-BEGIN_DECLS
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
 
 bool buffindexed_open(int mode);
 bool buffindexed_groupstats(const char *group, int *lo, int *hi, int *count,
@@ -26,6 +23,8 @@ bool buffindexed_expiregroup(const char *group, int *lo, struct history *h);
 bool buffindexed_ctl(OVCTLTYPE type, void *val);
 void buffindexed_close(void);
 
-END_DECLS
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
 
 #endif /* _BUFFINDEXED_H_ */
